#' @noRd
calculate_relative_abundance <- function(count_flag) {

  #' returns baseline data as list
  #' * start_year = start year used as baseline
  #' * year_range = first 10 years from start year
  #' * year_counts = first 10 years species counts
  #' * pval = log linear regression p value
  #' * baseline_count = predicted start_year count or mean 10 year count
  #' * baseline_method = method used to set baseline count either predicted start year or mean 10 year value
  #' * baseline_index = first year relative index value closest to start year
  #' * predicted_count = predicted first 10 year counts from significant regression or mean 10 year counts for non-significant regression
  #' * baseline = predicted first 10 year counts or mean 10 year counts divided by baseline_count
  function(ospar, label, species_name, target_threshold, start_year, end_year) {

    # ### KW: setting the baseline ###
    # ### GF updated to retain missing years and use latest year count as baseline for significant regression
    set_baseline <- function(x){

      # empty list to store baseline output
      baseline <- list()
      baseline$start_year <- start_year

      # get the first 10 year abundance counts
      x_sub <- subset(x, variable %in% start_year:(start_year + 9)) # first 10 years
      baseline$year_range <- x_sub$variable
      baseline$year_counts <- x_sub$value

      # run linear regression model
      # mod = lm(value~variable, data = x_sub)

      # MF: change to log linear regression, use quasipoisson
      # mod = glm(value~variable, data = x_sub, family = "poisson")
      mod = glm(value~variable, data = x_sub, family = "quasipoisson")

      # MF: save model plots summaries
      png(path("results", stringr::str_glue("{count_flag}_baseline_model"), species_name, stringr::str_glue("OSPAR{label}_residual.png")))
      plot(mod, 1) # residual scatter plot
      dev.off()
      png(path("results", stringr::str_glue("{count_flag}_baseline_model"), species_name, stringr::str_glue("OSPAR{label}_qq.png")))
      plot(mod, 2) # normal Q-Q plot
      dev.off()

      # extract the p-value from the model
      baseline$pval <- round(summary(mod)$coefficients[2, 4], 3)

      # GF modified code to use nearest year if start year missing or zero and log results
      if(baseline$pval <= 0.05) {
        # baseline <- purrr::detect(x_sub$value, function(x) {!is.na(x) & x != 0}) # use actual 1991 values
        # #set baseline = 1 if predicted 1991 value resulting from lm regression model below 1
        # if(baseline < 1) {baseline = 1}
        baseline$baseline_count <- predict(mod, x_sub, type = "response")[1] # use predicted 1991 value
        baseline$baseline_method <- str_glue("regression value for {start_year}")
        baseline$predicted_count <- predict(mod, x_sub, type = "response")
      } else {
        baseline$baseline_count <- mean(x_sub$value, na.rm = TRUE)
        baseline$baseline_method <- str_glue("mean {start_year}-{start_year + 9} value")
        baseline$predicted_count <- rep(baseline$baseline_count, 10)
      }
      baseline$baseline_index <- purrr::detect(x_sub$value, function(x) {!is.na(x) & x != 0}) / round(baseline$baseline_count, 5)
      baseline$baseline <- baseline$predicted_count / baseline$baseline_count
      luzlogr::printlog(stringr::str_glue("{species_name},{label},{count_flag},{baseline$pval},{baseline$baseline_method},{round(baseline$baseline_count, 5)},{baseline$baseline_index}"), ts = FALSE)
      return(baseline)
    }

    # ### GF: Set assessment to add to species trend plot
    #' returns assessment data as list
    #' * end_year = end year used as assessment
    #' * year_range = last 6 years from end year
    #' * relative_abundance = last 6 year relative abundance values
    #' * latest_relative_abundance_mean = latest relative abundance six year mean value
    #' * pval = linear regression p value
    #' * assessment = last 6 years predicted relative abundance value if regression significant
    #' * slope = last 6 years regression slope
    set_assessment <- function(x) {

      # empty list to store assessment output
      assessment <- list()
      assessment$end_year <- end_year

      # get the last 6 year abundance counts
      x_sub <- subset(x, years %in% (end_year - 5):end_year) # last 6 years
      assessment$year_range <- x_sub$years
      assessment$relative_abundance <- x_sub$index
      assessment$latest_relative_abundance_mean <- x_sub %>%
        dplyr::filter(years == end_year) %>%
        purrr::pluck(., "six_year_mean") %>%
        round(3)

      # Don't apply linear regression if 1 or 0 points in latest six years
      if (sum(!is.na(x_sub$index)) <= 1) {

        assessment$pval = "Not Applicable"
        assessment$slope = "Not Applicable"
        assessment$assessment <- rep(NA_integer_, 6)

      } else {

        # run log linear regression model
        #mod = glm(index~years, data = x_sub, family = "poisson")
        mod = glm(index~years, data = x_sub, family = "quasipoisson")

        # extract the p value
        assessment$pval <- round(summary(mod)$coefficients[2, 4], 3)

        # extract the slope
        assessment$slope <- round(summary(mod)$coefficients[2, 1], 3)

        # Don't apply log linear regression if p-val = NaN
        if (!is.nan(assessment$pval)) {
          assessment$assessment <- predict(mod, x_sub, type = "response")
        } else {
          assessment$pval = "Not Applicable"
          assessment$assessment <- rep(NA_integer_, 6)
        }
      }
      return(assessment)
    }

    plot_species_trend <- function() {

      # Set x axis range (rounded to nearest 5 years)
      xAxisMinYear <- plyr::round_any(start_year, 5)
      xAxisMaxYear <- plyr::round_any(end_year, 5, f = ceiling)

      # set y axis relative abundance index increased to nearest 2
      yAxisMaxIndex <- plyr::round_any(max(dat_unsm$index, na.rm = TRUE), 2, f = ceiling)

      # baseline data
      baseline_data <- tibble::tibble(
        year_range = baseline$year_range,
        baseline = baseline$baseline
      )

      # assessment line
      assessment_line <- tibble::tibble(
        year_range = assessment$year_range,
        line = assessment$assessment
      )

      # assessment point
      assessment_point <- tibble::tibble(
        year = assessment$end_year,
        six_year_mean = assessment$latest_relative_abundance_mean
      )

      # plot species relative abundance trend
      plot <- ggplot2::ggplot() +
        ggplot2::theme_classic() +
        ggplot2::theme(panel.background = ggplot2::element_rect(colour = "black"),
                       axis.text = ggplot2::element_text(face = "bold"),
                       plot.title = ggplot2::element_text(size = 10),
                       plot.subtitle = ggplot2::element_text(size = 8),
                       legend.position = "top",
                       legend.title = ggplot2::element_blank(),
                       legend.text = ggplot2::element_text(size = 8),
                       legend.margin = ggplot2::margin(0, 0, 0, 0),
                       legend.spacing = ggplot2::unit(0, "mm"),
                       legend.box.margin = ggplot2::margin(0, 0, 0, 0),
                       legend.box.spacing = ggplot2::unit(0, "mm"),
                       plot.margin = ggplot2::unit(c(0, 0, 0, 0), "mm")) +
        ggplot2::labs(title = stringr::str_glue("{stringr::str_to_title(stringr::str_replace(count_flag, '_', '-'))} {display_common_name(species_name)} in the {display_region_subdivision_names(label)}"),
                      subtitle = stringr::str_glue("trend: {baseline$start_year}-{baseline$start_year + 9}, p-value: {baseline$pval}
                                                   baseline: {baseline$baseline_method}
                                                   assessment: {assessment$end_year-5}-{assessment$end_year}, index = {assessment$latest_relative_abundance_mean}")) + #  p-value: {assessment$pval}, slope: {assessment$slope}
        ggplot2::coord_cartesian(xlim = c(xAxisMinYear, xAxisMaxYear),
                                 ylim = c(0, yAxisMaxIndex)) +
        ggplot2::scale_x_continuous(breaks = seq(xAxisMinYear, xAxisMaxYear, 5),
                                    name = NULL) +
        ggplot2::scale_y_continuous(breaks = c(0, 1, seq(0, yAxisMaxIndex, 0.05 * (10^(floor(log10(yAxisMaxIndex)) + 1)))), # handle differing y axis scales
                                    expand = c(0, 0),
                                    name = "relative abundance index") +
        ggplot2::scale_colour_manual(name = NULL,
                                     breaks = c("baseline", "six_year_mean"),
                                     values = c("darkgreen", "grey"),
                                     labels = c("log-linear trend estimate", "six year rolling geometric mean")) + # log-linear trend estimate
        ggplot2::scale_linetype_manual(name = NULL,
                                       breaks = c("baseline", "six_year_mean"),
                                       values = c("dashed", "solid"),
                                       labels = c("log-linear trend estimate", "six year rolling geometric mean")) + # log-linear trend estimate
        # add higher threshold target line of 1.3
        ggplot2::geom_hline(yintercept = 1.3, linetype = "dashed", size = 0.5) +
        # add lower threshold target line of either 0.8 (species lays one egg)
        ## or 0.7 (species lays more than one egg)
        ggplot2::geom_hline(yintercept = target_threshold, linetype = "twodash", size = 0.5) +
        # add baseline line
        ggplot2::geom_line(data = baseline_data,
                           mapping = ggplot2::aes(year_range, baseline, colour = "baseline", linetype = "baseline")) +
        # add relative abundance = 1 line
        ggplot2::geom_hline(yintercept = 1, linetype = "solid", size = 0.5) +
        # add six year rolling relative abundance index mean values against year

        ggplot2::geom_line(data = dat_unsm,
                           mapping = ggplot2::aes(years, six_year_mean, colour = "six_year_mean", linetype = "six_year_mean")) +
        # add assessment latest six year mean
        ggplot2::geom_point(data = assessment_point,
                            mapping = ggplot2::aes(year, six_year_mean),
                            colour = "darkblue",
                            size = 3) +
        # add assessment line
        # ggplot2::geom_line(data = assessment_line,
        #                    mapping = ggplot2::aes(year_range, line, colour = "assessment", linetype = "assessment")) +
        # add baseline first predicted point
        ggplot2::geom_point(data = baseline_data,
                            mapping = ggplot2::aes(year_range[1], baseline[1]),
                            colour = "darkgreen",
                            size = 3) +
        # add points plotting relative abundance index values against year
        ggplot2::geom_point(data = dat_unsm,
                            mapping = ggplot2::aes(years, index), colour = "darkred") +
        # add baseline text
        ggplot2::geom_text(data = baseline_data,
                           mapping = ggplot2::aes(year_range[1], baseline[1]),
                           nudge_y = 0.05 * yAxisMaxIndex,
                           colour = "darkgreen",
                           size = 3,
                           label = "baseline") +
        # add assessment text
        ggplot2::geom_text(data = assessment_point,
                           mapping = ggplot2::aes(year, six_year_mean),
                           nudge_y = 0.05 * yAxisMaxIndex,
                           nudge_x = -1,
                           colour = "darkblue",
                           size = 3,
                           label = "assessment")

      # save plot
      ggplot2::ggsave(plot = plot,
                      filename = path("results", stringr::str_glue("{count_flag}_out"), species_name, stringr::str_glue("OSPAR{label}_ALLDATA.png")),
                      width = 2100, height = 1400, units = "px")

    }

    if(nrow(ospar) > 0) {

      message(stringr::str_glue("Plotting trend for {species_name} in region {label}"))

      # convert abundance count to long format with site information and variable = year,
      ## value = count, with counts summed for each year
      ospar.melt = melt(ospar, id.vars = c("SiteID",
                                           "Colony",
                                           "SUBADMIN",
                                           "OSPAR_REGION",
                                           "OSPAR_SUBREGION"))

      # aggregates all the counts
      ## where all the subregion counts are passed into the function this results
      ## in the total region count
      ### eg Im, In, Io -> I
      ### eg IIa, IIb, IIc, IId, IIe, IIf -> II
      ospar.melt = aggregate(value ~ variable, data = ospar.melt, FUN = sum, na.rm = TRUE)
      ospar.melt$variable = as.numeric(as.character(ospar.melt$variable))
      ospar.melt = subset(ospar.melt, variable >= start_year & variable <= end_year)

      # add the missing years back as NA
      max_year_range <- full_seq(c(startYear, endYear), 1)
      ospar.melt <- ospar.melt %>%
        complete(variable = max_year_range)

      # set the baseline data
      baseline <- set_baseline(ospar.melt)
      baseline_count <- baseline$baseline_count

        # year range from start year to end years
        years = ospar.melt$variable

        # calculate relative abundance index based on baseline count
        index = ospar.melt$value / baseline_count

        # ### GF: calculating 6 year rolling geometric mean ###
        six_year_mean <- slider::slide_vec(index, function (x) {

          # Handling zeros in geometric mean, Marco: add 1 to all values before
          ## applying geometric mean, then take 1 of result
          ## see https://www.wwdmag.com/channel/casestudies/handling-zeros-geometric-mean-calculation
          #      https://arxiv.org/pdf/1806.06403.pdf#:~:text=By%20definition%2C%20the%20geometric%20mean,about%20the%20non%2Dzero%20values.
          zero_present <- FALSE
          if(purrr::has_element(x, 0)) {
            x <- x + 1
            zero_present <- TRUE
          }

          ## calculate geometric mean
          ## https://stackoverflow.com/questions/2602583/geometric-mean-is-there-a-built-in
          exp(mean(log(x), na.rm = TRUE)) - zero_present
        },
        .before = 5, .complete = TRUE)

        # predicted relative abundance yearly values taken from linear model
        dat_unsm = data.frame(years, index, six_year_mean)

        # get assessment values for plot
        assessment <- set_assessment(dat_unsm)

        # plot species trend
        plot_species_trend()

        # return relative abundance and geometric mean relative abundance values
        dat_unsm$region <- stringr::str_glue("OSPAR{label}")
        dat_unsm$species <- species_name
        dat_unsm <- dat_unsm[, c("region", "species", "years", "index", "six_year_mean")]
    }
  }
}

#' @title
#' Calculate and plot species breeding relative abundance
#'
#' @description
#' This function calculates relative abundance, six year relative abundance geometric
#' mean and plots relative abundance trend for breeding species.
#'
#' @details
#' The function performs the following steps
#'
#' * aggregates abundance counts as value for each year (variable) in data frame **ospar.melt**.
#' Aggregation required for OSPAR regions with sub region counts eg I <- Io, Im, In
#' * runs step baseline function to get the predicted start year count from first ten
#' years regression model or mean of the first ten year if regression is not signifcant
#' * calculates the relative abundance for each year by dividing count by baseline count,
#' stored in **index** vector
#' * calculates the six year rolling geometric relative abundance means, stored in
#' **six_year_mean** vector
#' * creates **dat_unsm** dataframe containing years, index and six_year_mean columns
#' * creates and exports species yearly relative abundance trend plot for each species+region/sub region
#' * returns yearly and six year mean relative abundance values for each species+region/sub region as
#' **dat_unsm** data frame.
#'
#' @family calculate relative abundance functions
#'
#' @param ospar dataframe of OSPAR region or sub region aggregated yearly abundance counts.
#' @param label OSPAR region or sub region label. One of I, Im, In, Io, II, IIa, IIb, IIc, IId IIe, IIf, III, IV, V.
#' @param species_name Common name of species
#' @param target_threshold Target threshold for species, 0.7 if species lays more than egg, 0.8 for species laying one egg.
#' @param start_year Start year of abundance analysis
#' @param end_year End year of abundance analysis.
#' @param count_flag Are the abundance counts breeding or non breeding counts.
#'
#' @return list containing region or subregions yearly and six year relative abundance means,
#' for each breeding species.
#'
#' @export
calculate_breeding_relative_abundance <- calculate_relative_abundance(count_flag = "breeding")

#' @title
#' Calculate and plot species non-breeding relative abundance
#'
#' @description
#' This function calculates relative abundance, six year relative abundance geometric
#' mean and plots relative abundance trend for non-breeding species.
#'
#' @inherit calculate_breeding_relative_abundance return details
#'
#' @family calculate relative abundance functions
#'
#' @inheritParams calculate_breeding_relative_abundance
#'
#' @return list containing region or subregions yearly and six year relative abundance means,
#' for each non-breeding species.
#'
#' @export
calculate_non_breeding_relative_abundance <- calculate_relative_abundance(count_flag = "non_breeding")

#' @noRd
latest_assessment_plot <- function(region_list, count_flag, title,
                                   assessment_year = 2021) {

  function(latest.dat = ByRegion.dat.latest.mean, previous.dat = ByRegion.dat.previous.mean) {

    # create assessment matrix
    create_assessment_matrix <- function(.dat, region_list) {
      as.matrix(.dat[, region_list])
    }

    # Colour matrix, **colour.mat** created from ByRegion.mat matrix,
    # comparing latest six year relative abundance mean values to species
    ## threshold and upper threshold
    create_colour_matrix <- function(.mat, .dat) {

      # create empty colour matrix
      colour.mat = matrix(data = 0, ncol = ncol(.mat), nrow = nrow(.mat))

      for(i in 1:ncol(colour.mat)){
        for(j in 1:nrow(colour.mat)){

          if(.dat$Eggs[j] == 1) target <- 0.8
          if(.dat$Eggs[j] > 1) target <- 0.7

          ifelse(.mat[j, i] < target,
                 colour.mat[j, i] <- colBelowThreshold,
                 ifelse(.mat[j, i] >= 1.3,
                        colour.mat[j,i] <- colaboveUpperThreshold,
                        ifelse(.mat[j, i] >= target & .mat[j, i] < 1.3,
                               colour.mat[j,i] <- colAboveThreshold ,
                               colour.mat[j,i] <- colIncompleteInfo)))
        }
      }

      # give colour for incomplete information where relative abundance value = "0"
      colour.mat[colour.mat == "0"] <- colIncompleteInfo
      colnames(colour.mat) <- colnames(latest.mat)
      colour.mat
    }

    # create_change_matrix
    create_change_matrix <- function(colour.latest.mat, colour.previous.mat) {

      # convert colour to numeric matrix
      colour_to_numeric_matrix <- function(.mat) {

        # convert colour matrix to numeric values
        .mat[.mat == colaboveUpperThreshold] <- 3L
        .mat[.mat == colAboveThreshold] <- 2L
        .mat[.mat == colBelowThreshold] <- 1L
        .mat[.mat == colIncompleteInfo] <- NA_integer_
        .mat <- matrix(as.numeric(.mat), ncol = ncol(.mat))
      }

      # calculate change matrix
      numeric.latest.mat <- colour_to_numeric_matrix(colour.latest.mat)
      numeric.previous.mat <- colour_to_numeric_matrix(colour.previous.mat)
      change.mat <- numeric.latest.mat - numeric.previous.mat
    }

    # add functional group label
    add_functional_group_label <- function(labels = functional_group_labels,
                                           functional_group) {

      # Get line and label position on y axis
      min_y_position <- min(labels[labels$Fun_Group == functional_group, ]$row - 1)
      max_y_position <- max(labels[labels$Fun_Group == functional_group, ]$row)
      mid_y_position <- mean(labels[labels$Fun_Group == functional_group, ]$row)

      # adjust Grazing Feeder text vertical position
      if(functional_group == "Grazing Feeder") {
        mid_y_position <- mid_y_position - 1
      }

      # change functional groups to multiple lines
      if(functional_group == "Water Column Feeder") {
        functional_group = "Water Column \n Feeder"
      }

      if(functional_group == "Wading Feeder") {
        functional_group = "Wading \n Feeder"
      }

      if(functional_group == "Surface Feeder") {
        functional_group = "Surface \n Feeder"
      }

      if(functional_group == "Grazing Feeder") {
        if(count_flag == "breeding") {
          functional_group = "Gr. \n Feed"
        } else {
          functional_group = "Grazing \n Feeder"
        }
      }

      if(functional_group == "Benthic Feeder") {
        if(count_flag == "breeding") {
          functional_group = "Be. \n Feed."
        } else {
          functional_group = "Benthic \n Feeder"
        }
      }

      # add functional group line to rows
      axis(side = 2,
           at = c(min_y_position, max_y_position),
           line = 14.5,
           labels = c("",""))

      # add function group label to rows
      text(x = -2.2,
           y = mid_y_position,
           labels = functional_group,
           xpd = TRUE,
           srt = 90,
           cex = 1)
    }

    # Colour palette (red, orange, green, gray)
    colBelowThreshold <- "red"
    colAboveThreshold <- "green"
    colaboveUpperThreshold <- "darkgreen"
    colIncompleteInfo <- "gray"

    # create latest assessment matrix
    latest.mat <- create_assessment_matrix(latest.dat, region_list)

    # create latest colour matrix
    colour.latest.mat <- create_colour_matrix(latest.mat, latest.dat)

    # create previous assessment matrix
    previous.mat <- create_assessment_matrix(previous.dat, region_list)

    # create previous colour matrix
    colour.previous.mat <- create_colour_matrix(previous.mat, previous.dat)

    # create change matrix
    change.mat <- create_change_matrix(colour.latest.mat, colour.previous.mat)

    # set graphic device
    png(path("results", stringr::str_glue("{count_flag}_out"),
             stringr::str_glue("{title}_{assessment_year}.png")),
        width = (ncol(colour.latest.mat) * 100) + 500,
        height = nrow(colour.latest.mat) * 20)

    # set margins
    par(mar = c(4, 30, 4, 4))

    # create plot
    plotrix::color2D.matplot(latest.mat,
                             cellcolors = colour.latest.mat,
                             show.values = FALSE,
                             axes = FALSE,
                             xlab = "", ylab = "")

    # add species names to rows
    axis(side = 2,
         at = seq(1, length(latest.dat$species), 1),
         labels = display_common_name(rev(latest.dat$species)),
         las = 2,
         tick = FALSE,
         cex.axis = 1.2,
         padj = 1.5,
         font = 4)

    # add regions names to columns
    axis(side = 3,
         at = c(1:ncol(colour.latest.mat)),
         labels = region_list,
         tick = FALSE,
         cex.axis = 1,
         font = 2,
         padj = 0.5,
         hadj = 1.2)

    # add zero to end of rows required for the producing arrows for the first row
    rows = rev(seq(0, nrow(change.mat), 1))

    # add arrows indicating change based on comparing latest threshold status with
    # previous threshold status
    for (i in 1:nrow(change.mat)){
      for (j in 1:ncol(change.mat)){

        # change above upper threshold to below species threshold: vertical thick arrow pointing down
        if(!is.na(change.mat[i,j]) & change.mat[i,j] == -2)
          arrows(x0 = j-0.5, y0 = rows[i], x1 = j-0.5, y1 = rows[i+1], length = 0.15, lwd = 2)

        # change above upper threshold to between threshold or between thresholds to below species threshold:
        # vertical thin arrow pointing down
        if(!is.na(change.mat[i,j])& change.mat[i,j] == -1)
          arrows(x0 = j-0.5, y0 = rows[i], x1 = j-0.5, y1=rows[i+1], length = 0.15, lwd = 1)

        # no change between thresholds
        # if(!is.na(change.mat[i,j]) & change.mat[i,j] == 0)
        #   arrows(x0 = j-0.75, y0 = rows[i]-0.5, x1 = j-0.25, y1 = rows[i] - 0.5, length = 0.15, lwd = 1, code = 3)

        # change below species threshold to between thresholds or between thresholds to above upper thresholds:
        # vertical thin arrow pointing up
        if(!is.na(change.mat[i,j]) & change.mat[i,j] == 1)
          arrows(x0 = j-0.5, y0 = rows[i+1], x1 = j-0.5, y1 = rows[i], length = 0.15, lwd = 1)

        # change below threshold to above upper threshold: vertical thick arrow pointing up
        if(!is.na(change.mat[i,j]) & change.mat[i,j] == 2)
          arrows(x0 = j-0.5, y0 = rows[i+1], x1 = j-0.5, y1 = rows[i], length = 0.15, lwd = 2)
      }
    }

    # add functional group labels
    functional_group_labels <- latest.dat[nrow(latest.dat):1, ]
    functional_group_labels$row <- seq.int(nrow(functional_group_labels))

    for(i in unique(functional_group_labels$Fun_Group)) {
      add_functional_group_label(functional_group = i)
    }

    # turn off graphic device
    dev.off()
  }
}

#' @title
#' Plot the latest breeding assessment species statuses for all OSPAR regions
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for each OSPAR region, adding trend arrows
#' corresponding to changes in threshold status between a previous and latest six year geometric means.
#'
#' @details
#  Different colours in the plot indicates whether six year relative abundance mean
#'
#' * above upper relative abundance threshold of 1.3.
#' * above species threshold set by number of eggs species lays (threshold 0.7 or 0.8)
#' * below species threshold (threshold 0.7 or 0.8)
#' * incomplete information
#'
#' Different arrows in the plot indicate change in threshold status
#'
#' * vertical thick arrow pointing down for change from above upper threshold to below species threshold.
#' * vertical thin arrow pointing down for either change from above upper threshold to between threshold
#' or between thresholds to below species threshold.
#' * vertical thin arrow pointing up for either change from below species threshold to between thresholds
#' or between thresholds to above upper threshold.
#' * vertical thick arrow pointing up for change below threshold to above upper threshold.
#'
#' @family calculate latest assessment plot functions
#'
#' @param latest.dat data frame containing species latest six year geometric mean
#' for each region
#' @param change.dat data frame containing species latest six year yearly percentage
#' relative abundance change
#'
#' @return latest breeding assessment plot for each species within each OSPAR region
#'
#' @export
latest_breeding_assessment_plot_OSPAR_ALL <- latest_assessment_plot(
  region_list = c("OSPARI", "OSPARII", "OSPARIII", "OSPARIV"),
  count_flag = "breeding",
  title = "AllRegions_breeding")

#' @title
#' Plot the latest breeding assessment species statuses for OSPAR I region and sub regions.
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for OSPAR I region and sub regions.
#'
#' @inherit latest_breeding_assessment_plot_OSPAR_ALL return details
#'
#' @family calculate latest assessment plot functions
#'
#' @inheritParams latest_breeding_assessment_plot_OSPAR_ALL
#'
#' @param data.dat data frame containing species latest six year geometric mean
#' for each region
#'
#' @return latest breeding assessment plot for each species within each OSPAR region
#'
#' @export
latest_breeding_assessment_plot_OSPARI <- latest_assessment_plot(
  region_list = c("OSPARI", "OSPARIm", "OSPARIn", "OSPARIo"),
  count_flag = "breeding",
  title = "RegionOSPARI_breeding")

#' @title
#' Plot the latest breeding assessment species statuses for OSPAR II region and sub regions.
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for OSPAR II region and sub regions.
#'
#' @inherit latest_breeding_assessment_plot_OSPAR_ALL return details
#'
#' @family calculate latest assessment plot functions
#'
#' @inheritParams latest_breeding_assessment_plot_OSPAR_ALL
#'
#' @param data.dat data frame containing species latest six year geometric mean
#' for each region
#'
#' @return latest breeding assessment plot for each species within each OSPAR region
#'
#' @export
latest_breeding_assessment_plot_OSPARII <- latest_assessment_plot(
  region_list = c("OSPARII", "OSPARIIa", "OSPARIIb", "OSPARIIc", "OSPARIId", "OSPARIIe", "OSPARIIf"),
  count_flag = "breeding",
  title = "RegionOSPARII_breeding")

#' @title
#' Plot the latest wintering assessment species statuses for all OSPAR regions
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for each OSPAR region.
#'
#' @inherit latest_breeding_assessment_plot_OSPAR_ALL return details
#'
#' @family calculate latest assessment plot functions
#'
#' @inheritParams latest_breeding_assessment_plot_OSPAR_ALL
#'
#' @return latest wintering assessment plot for each species within each OSPAR region
#'
#' @export
latest_wintering_assessment_plot_OSPAR_ALL <- latest_assessment_plot(
  region_list = c("OSPARI", "OSPARII", "OSPARIII"),
  count_flag = "non_breeding",
  title = "AllRegions_wintering")

#' @title
#' Plot the latest wintering assessment species statuses for OSPAR I region and sub regions.
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for OSPAR I region and sub regions.
#'
#' @inherit latest_breeding_assessment_plot_OSPAR_ALL return details
#'
#' @family calculate latest assessment plot functions
#'
#' @inheritParams latest_breeding_assessment_plot_OSPAR_ALL
#'
#' @param data.dat data frame containing species latest six year geometric mean
#' for each region
#'
#' @return latest wintering assessment plot for each species within each OSPAR region
#'
#' @export
latest_wintering_assessment_plot_OSPARI <- latest_assessment_plot(
  region_list = c("OSPARI", "OSPARIn", "OSPARIo"),
  count_flag = "non_breeding",
  title = "RegionOSPARI_wintering")

#' @title
#' Plot the latest wintering assessment species statuses for OSPAR II region and sub regions.
#'
#' @description
#' This function plots the species latest six year geometric relative abundance mean status
#' against the relative abundance threshold for OSPAR II region and sub regions.
#'
#' @inherit latest_breeding_assessment_plot_OSPAR_ALL return details
#'
#' @family calculate latest assessment plot functions
#'
#' @inheritParams latest_breeding_assessment_plot_OSPAR_ALL
#'
#' @param data.dat data frame containing species latest six year geometric mean
#' for each region
#'
#' @return latest wintering assessment plot for each species within each OSPAR region
#'
#' @export
latest_wintering_assessment_plot_OSPARII <- latest_assessment_plot(
  region_list = c("OSPARII", "OSPARIIa", "OSPARIIb", "OSPARIIc", "OSPARIId", "OSPARIIe", "OSPARIIf"),
  count_flag = "non_breeding",
  title = "RegionOSPARII_wintering")

#' @noRd
latest_proportion_species_above_threshold <- function(count_flag) {

  function(latest.dat, region, functional_group) {

    # filter on functional group if supplied
    if (functional_group != "All Feeders") {
      latest.dat <- latest.dat[latest.dat$Fun_Group == functional_group, ]
    }

    # count total number of species with latest six year relative abundance mean score
    total <- length(which(!is.na(latest.dat[, 5])))

    # count the number of species exceeding 0.7 threshold for species laying more than 1 egg +
    # the number of species exceeding 0.8 threshold for species laying 1 egg
    pass <- length(which(latest.dat[, 5] > 0.7 & latest.dat$Eggs > 1)) +
      length(which(latest.dat[, 5] > 0.8 & latest.dat$Eggs == 1))

    # Calculate the percentage number of species passing the threshold
    proportion <- round(pass / total * 100, 1)

    # Add row of percentage threshold stats to data frame storing results
    list(ospar = region,
         countflag = count_flag,
         functionalGroup = functional_group,
         totalSpecies = total,
         passSpecies = pass,
         percentSpecies = proportion)
  }
}

#' @title
#' Latest proportion of breeding functional group species above threshold
#'
#' @description
#' This function calculates the proportion of breeding species within a functional group and
#' OSPAR region, whose latest six year relative abundance mean is above the threshold
#' for that species. The threshold is 0.7 for species laying more than one egg and
#' 0.8 for species laying one egg.
#'
#' @family latest proportion above threshold
#'
#' @param latest.dat data frame containing the latest six year mean relative abundance value for each species
#' @param region OSPAR region label, one of "I", "II", "III", "IV", "V"
#' @param functional_group species functional group, one of "All Feeders", "Wading Feeder",
#' "Surface Feeder", "Water Column Feeder", "Benthic Feeder", "Grazing Feeder"
#'
#' @return list containing ospar region label, count flag, functional group, total number of species,
#' total number of species above threshold, percentage of species above threshold
#'
#' @export
latest_proportion_breeding_species_above_threshold <- latest_proportion_species_above_threshold(count_flag = "breeding")

#' @title
#' Latest proportion of wintering functional group species above threshold
#'
#' @description
#' This function calculates the proportion of wintering species within a functional group and
#' OSPAR region, whose latest six year relative abundance mean is above the threshold
#' for that species. The threshold is 0.7 for species laying more than one egg and
#' 0.8 for species laying one egg.
#'
#' @family latest proportion above threshold
#'
#' @inheritParams latest_proportion_breeding_species_above_threshold
#'
#' @return list containing ospar region label, count flag, functional group, total number of species,
#' total number of species above threshold, percentage of species above threshold
#'
#' @export
latest_proportion_wintering_species_above_threshold <- latest_proportion_species_above_threshold(count_flag = "non-breeding")







