#' @title
#' Seabird species lookup table
#'
#' @description
#' List of seabirds used in BH1 breeding and wintering abundance counts analysis,
#' along with their details
#'
#' @details
#' The following changes were made to the common names to allow them to be used in the analysis
#'
#' * Apostrophe removed in Bewick's swan
#' * Apostrophe removed in Cory's shearwater
#' * Apostrophe removed in Leach's storm-petrel
#' * Apostrophe removed in Steller's eider
#' * Unicode character and apostrophe removed in Brünnich's guillemot
#'
#' @format A tibble with `r nrow(species)` rows and `r ncol(species)` variables
#' \describe{
#'   \item{aphia_id}{AphiaID of species in the World Register of Marine Species (WoRMS)}
#'   \item{scientific_name}{Species scientific name, listed in Annex 2 CEMP guidelines 20220228}
#'   \item{common_name}{Species common name, listed in Annex 2 CEMP guidelines 20220228}
#'   \item{eggs}{Number of eggs laid in breeding season}
#'   \item{functional_group}{
#'
#'   One of five seeding groups assigned to species
#'
#'   * Benthic Feeder
#'   * Grazing Feeder
#'   * Surface Feeder
#'   * Wading Feeder
#'   * Water Column Feeder
#'
#'   }
#'   \item{required_count_unit}{Whether individuals or pairs are required to be counted
#'   for the species}
#'   \item{conversion}{Method used to convert counts to required counting unit}
#' }
"species"

#' @title
#' Country lookup table
#'
#' @description
#' Country names and two letter ISO codes submitting bird colony abundance counts
#'
#' @format A tibble with `r nrow(country)` rows and `r ncol(country)`
#'  variables
#' \describe{
#'   \item{iso_country_code}{Two letter ISO country code}
#'   \item{country_name}{Name of country or British Crown Dependency}
#' }
"country"

#' @title
#' Regional weightings lookup table
#'
#' @description
#' Regional weightings for OSPAR regions where the entire bird species populations
#' are not monitored.
#'
#' @details
#' Source year weighting range values have been converted to a single middle
#' year (or nearest latest year to middle), with missing values removed.
#'
#' Weighting values have been standardized to either pair or individual breeding
#' counts as given in the required_species_count field in the species table and
#' zero weighting values have been removed.
#'
#' @format A tibble with `r nrow(regional_weightings)` rows and `r ncol(regional_weightings)`
#'  variables
#' \describe{
#'   \item{country}{Country's ISO 3166 2 digit code}
#'   \item{aphia_id}{AphiaID of species in the World Register of Marine Species (WoRMS)}
#'   \item{species_name}{Species scientific name, , according  to the World Register of Marine Species (WoRMS)}
#'   \item{ospar_region}{
#'
#'    OSPAR region number
#'
#'   * 1 = Arctic Waters (OSPAR region I)
#'   * 2 = Greater North Sea (OSPAR region II)
#'   * 3 = Celtic Seas (OSPAR region III)
#'   * 4 = Bay of Biscay and Iberian Coast (OSPAR region IV)
#'   * 5 = Wider Atlantic (OSPAR region V)
#'
#'   }
#'   \item{ospar_subregion}{specific bird subdivision within OSPAR regions I or II}
#'   \item{colony_ices_label}{Label used for OSPAR region and birds subdivision}
#'   \item{weighting_value}{}
#'   \item{source_year_weighting}{}
#'   \item{count_unit}{
#'
#'   Method used to count birds, breeding birds are either counts of individuals or
#'   pairs, non breeding (wintering) bird counts are counts of individuals
#'
#'   * 1 = number of individuals
#'   * 2 = pairs
#'
#'   }
#'   \item{count_flag}{Flag indicating if the count was breeding data or non breeding data (overwintering or migratory birds)}
#' }
"regional_weightings"

#' @title
#' Species 2021 count status in each OSPAR region / sub-region
#'
#' @description
#' This table lists the data status for the 2021 data for each breeding and wintering
#' species in each OSPAR region / sub-region.
#'
#' @details
#' Each species within each OSPAR region / sub-region was assigned one of three
#' data statuses, based on the sum of the counts for each year with the year range used
#' within the analysis and review of species trend graphs by bird experts.
#'
#' Following this review of species trends by bird experts in March 2022, each species region / sub-region were
#' flagged as either to be included or excluded in the analysis, with the following three data statuses assigned.
#'
#' * absent - counts are all zero or missing in the year range.
#' * exclude - insufficient counts within the first 10 years between 1991-2020 (2 counts or less, or all zero counts),
#' last 6 years or too many counts are missing in the year range for the analysis, or exclusion recommended following
#' review of species trend plots by bird experts
#' * analysis - sufficient years in the year range for the analysis in the OSPAR region / sub-region are present and
#' no issue found with species trend plots following review by bird experts.
#'
#' @format A tibble with `r nrow(count_status)` rows and `r ncol(count_status)`
#'  variables
#' \describe{
#'   \item{id}{id number of the breeding or wintering species in each OSPAR region}
#'   \item{count_flag}{breeding or non-breeding wintering counts}
#'   \item{ospar}{OSPAR region / sub-region}
#'   \item{common_name}{Common name of species}
#'   \item{data}{Status of counts for the species in the year range used in the analysis}
#'  }
"count_status"
