library(b1indicator)

# stop scientific notation display
options(scipen = 999)

# set config
Sys.setenv(R_CONFIG_ACTIVE = "production")

# network_paths
ices_data_path <- config::get("ices_data_path")
colony_path <- config::get("colony_path")

## corrected ices colony data from first download ------------------------------
colony_ices_corrected <- readr::read_delim(fs::path(colony_path, "Birds_Colonies_Corrected_20211116.csv"),
                                       delim = ",") %>%
  readr::stop_for_problems() %>%
  janitor::clean_names() %>%
  dplyr::rename(ices_calculated_ospar_birds_subdivision = ices_calculated_ospar_birds_sub_division,
                birds_subdivision = birds_sub_division,
                area_verified = region_v3,
                sub_region_verified = sub_division_v3,
                latitude =  lat,
                longitude = lon) %>%
  dplyr::mutate(sub_region_verified = stringr::str_remove(sub_region_verified, "[:digit:]")) %>%
  dplyr::mutate(dplyr::across(tidyselect::everything(), dplyr::na_if, "NULL")) %>%
  dplyr::mutate(dplyr::across(tidyselect::everything(), dplyr::na_if, "")) %>%
  dplyr::select(-id, -sub_division_v2)

## remove ices colonies from Portugal (tbl_upload_id = 1169) and Iceland (tbl_upload_id = 2184)
colony_ices_corrected <- colony_ices_corrected %>%
  dplyr::filter(!tbl_upload_id %in% c(1169, 2184))

## add ices colonies from Portugal (tbl_upload_id = 1169) and Iceland (tbl_upload_id = 2184) from new download
colony_ices_add <- readr::read_delim(fs::path(colony_path,
                                          "Birds_18112021_Extration",
                                          "tblvw_BirdsColony_18112021.csv"),
                                 delim = ",") %>%
  readr::stop_for_problems() %>%
  janitor::clean_names() %>%
  dplyr::rename(ices_calculated_ospar_birds_subdivision = ices_calculated_ospar_birds_sub_division,
                birds_subdivision = birds_sub_division,
                area_verified = area_reference,
                sub_region_verified = birds_sub_division,
                latitude =  lat,
                longitude = lon) %>%
  dplyr::mutate(sub_region_verified = stringr::str_remove(sub_region_verified, "[:digit:]")) %>%
  dplyr::mutate(dplyr::across(tidyselect::everything(), dplyr::na_if, "NULL")) %>%
  dplyr::mutate(dplyr::across(tidyselect::everything(), dplyr::na_if, "")) %>%
  dplyr::filter(tbl_upload_id %in% c(1169, 2184))

# append Portugal and Iceland colonies
colony_ices <- dplyr::bind_rows(colony_ices_corrected, colony_ices_add)

# correct colony regions and sub regions
colony_ices <- colony_ices %>%
  dplyr::mutate(sub_region_verified = dplyr::case_when(
    # colonies Io to In
    ices_colony_id %in% c(92170L, 92194L, 92715L, 92605L, 92283L) ~ "n",
    # add subregion letter for Arctic Waters
    area_verified == 1L & is.na(sub_region_verified) ~ "p",
    # colonies IIa to IIf
    ices_colony_id %in% c(104258L, 111306L, 112002L, 111533L) ~ "f",
    # colonies IIa to IId
    ices_colony_id %in% c(107545L, 107546L, 107547L, 107548L, 107549L, 107550L,
                          107551L, 107552L, 107553L, 107554L, 107555L, 107556L, 107557L) ~ "d",
    # colony IIb to Io
    ices_colony_id %in% c(92362L) ~ "o",
    # colony IIc to IId
    ices_colony_id %in% c(113008L, 113009L) ~ "d",
    # colony IId to IIe
    ices_colony_id %in% c(110647L, 112340L, 109954L) ~ "e",
    # colony IIe to IId
    ices_colony_id %in% c(101531L, 101532L, 101533L, 101534L, 101535L, 101536L,
                            101537L, 101538L, 101539L, 101540L, 101541L, 101542L,
                            101543L, 101544L, 101545L, 101546L, 101547L, 101548L,
                            101549L, 101550L, 101551L, 101552L, 101553L, 101554L,
                            101555L, 101556L, 101557L, 101582L, 101583L, 101584L,
                            101585L, 101586L, 101587L) ~ "d",
    # colony IIf to IIa
    ices_colony_id %in% c(110629L) ~ "a",
    # colony III to IIa
    ices_colony_id %in% c(104345L) ~ "a",
    # colony III to IIe
    ices_colony_id %in% c(111624L) ~ "e",
    # colony III to IIf
    ices_colony_id %in% c(108279L, 108791L) ~ "f",
    TRUE ~ sub_region_verified
  )) %>%
  dplyr::mutate(area_verified = dplyr::case_when(
    ices_colony_id %in% c(92362L) ~ 1,
    ices_colony_id %in% c(111624L, 108279L, 108791L, 104345L) ~ 2,
    TRUE ~ area_verified
  ))

# correct Arctic Waters longitude
colony_ices <- colony_ices %>%
  dplyr::mutate(longitude = dplyr::case_when(
    area_verified == 1L & sub_region_verified == "p" ~ longitude * -1,
    TRUE ~ longitude
))

## add OSPAR label
colony_ices <- colony_ices %>%
  dplyr::mutate(colony_ices_label = stringr::str_glue("{as.roman(area_verified)}{tidyr::replace_na(sub_region_verified, '')}"))

## remove unwanted columns
colony_ices <- colony_ices %>%
  dplyr::select(ices_colony_id:colony_name, latitude, longitude, sub_region_verified:colony_ices_label) %>%
  dplyr::rename(area_reference = area_verified,
                birds_subdivision = sub_region_verified) %>%
  dplyr::relocate(area_reference, .after = colony_name) %>%
  dplyr::relocate(birds_subdivision, .after = area_reference) %>%
  dplyr::relocate(colony_ices_label, .after = birds_subdivision)

## check and correct colony details

### Validation: ices_colony_id should be unique
colony_ices %>%
  assertr::assert(assertr::is_uniq, ices_colony_id,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

### Validation: ices_colony_id should uniquely identify colonies
### No column appears to identify duplicate colonies in ICES database.
#### ices_colony_id - ids unique including duplicate colonies
#### latitude, longitude-colonies with identical latitude, longitude have different names and national colony ids
#### national_colony_id - colonies with identical ids have different names from different countries
#### This fails: No action required
colony_ices %>%
  dplyr::distinct(ices_colony_id, country, colony_name) %>%
  janitor::get_dupes(country, colony_name) %>%
  assertr::verify(nrow(.) == 0,
                  success_fun = assertr::success_logical, error_fun = assertr::just_warn)

### Validation: Colonies should be in the correct OSPAR area_reference and birds_subdivision.
#### unable to check using spatial intersection with ospar boundaries as colonies
#### occurring inland are not contained within the ospar boundary polygons.
#### manually check each ospar region and birds subdivision against ospar boundaries
#### Spatial check using colonies latitude and longitude
fs::dir_create(fs::path(tempdir(), "shapefiles"))

colony_ices_spatial <- colony_ices %>%
  dplyr::select(-area_reference, -birds_subdivision) %>%
  sf::st_as_sf(coords = c("longitude", "latitude"), crs = 4326)

colony_ices_spatial %>%
  dplyr::group_nest(colony_ices_label, .key = "colony") %>% {
    purrr::walk2(.$colony, .$colony_ices_label,
                 function(.x, .y) {
                   sf::st_write(.x, fs::path(fs::path(tempdir(), "shapefiles", stringr::str_glue("{.y}.shp"))))
                 })}

# save local version of colony_ices dataset
pins::pin_remove("colony_ices", board = "local")
pins::pin(colony_ices, "colony_ices")

## ices abundance data ---------------------------------------------------------

# first download - 20211001
# second download - 20211029
## removed all Denmark breeding data IIc, IId (ICES_field_id = 98)
## added additional Norway breeding data Im, Io (ICES_field_id = 2180)
## added additional Denmark non breeding data IIc (ICES_field_id = 2181)
# third download - 20211114
## added Denmark breeding data IIc, IId (ICES_field_id = 2182, 2183)
# fourth download - 20211118
## added Portugal and Iceland breeding data (ICES_field_id = 1169, 2184)
b1_abundance_data_ices <- readr::read_delim(fs::path(ices_data_path,
                                                     "Birds_18112021_Extration",
                                                     "tblvw_BirdsAbundance_18112021.csv"),
                                            col_types = readr::cols(...1 = readr::col_skip(),
                                                                    preparationDate = readr::col_skip()),
                                            delim = ",") %>%
  readr::stop_for_problems() %>%
  janitor::clean_names() %>%
  dplyr::mutate(plot_combination = as.integer(plot_combination),
                sample_breeding = as.integer(sample_breeding)) %>%
  dplyr::mutate(dplyr::across(tidyselect::everything(), dplyr::na_if, "NULL"))

## ----- year -----
#### Validation: Abundance counts are between 1939 and 2020
b1_abundance_data_ices %>%
  assertr::assert(assertr::within_bounds(1939, 2020), year,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

### exclude counts before 1984 and after 2020
#### take counts back to earliest source year weighting, non-breeding Norway counts 1984
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::filter(dplyr::between(year, 1984, 2020))

## ----- survey details -----
b1_abundance_survey_ices <- b1_abundance_data_ices %>%
  dplyr::select(country, survey_name, data_access) %>%
  dplyr::distinct() %>%
  dplyr::arrange(survey_name, country, data_access)

### remove survey details
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::select(-tidyselect::starts_with("survey"),
                -data_access)

## ----- count -----
#### Validation: Abundance counts are between -1 and 549,235
b1_abundance_data_ices %>%
  assertr::assert(assertr::within_bounds(-1, 549235), count,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

### exclude no data counts having -1
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::filter(count != -1)

### ----- remove unwanted fields -----
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::select(-ices_file_id,
                -country,
                -tidyselect::starts_with("ices_calculated"),
                -(colony_name:location_definition),
                -national_colony_id,
                -species_name)

## ----- ices_colony_id, country, colony_name, area_reference, birds_subdivision, latitude, longitude -----
### use ICES colony OSPAR and birds subdivision regions
colony_ices <- pins::pin_get("colony_ices")

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::left_join(colony_ices, by = "ices_colony_id")

#### Validation: All abundance data should have been matched to a colony
b1_abundance_data_ices %>%
  assertr::assert(assertr::not_na, country, area_reference,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

## ----- aphia_id, scientific_name, common_name -----

### remove aphia_id = 2 (Animalia)
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::filter(aphia_id != 2)

### Use species look up
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::left_join(species %>%
                     dplyr::select(aphia_id, tidyselect::contains("name")),
                   by = "aphia_id")

#### Validation: All abundance data should have been matched to a species
b1_abundance_data_ices %>%
  assertr::assert(assertr::not_na, scientific_name, common_name,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

## ----- sample_breeding -----
### sample breeding indicates either plot counts = 1 or whole colony count = 2

## All Ireland colonies are missing sample breeding, updated as all colony counts
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(sample_breeding = dplyr::case_when(
    country == "IE" ~ 2L,
    TRUE ~ sample_breeding
  ))

## All missing non-breeding counts updated as colony counts
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(sample_breeding = dplyr::case_when(
    count_flag == "non_breeding" & is.na(sample_breeding) ~ 2L,
    TRUE ~ sample_breeding
  ))

#### Validation: All counts should have sample breeding 1 or 2
#### This fails as sample_breeding counts are missing from
##### 2013 NL breeding counts
b1_abundance_data_ices %>%
  assertr::assert(assertr::in_set(1, 2, allow.na = FALSE), sample_breeding,
                  success_fun = assertr::success_logical, error_fun = assertr::just_warn)

## ----- plot_combination -----
### plot combination is the identity of each different combination of plots where counts
### are based on plots and not the whole colony. Values = “1”, “2”, “3”, “4”, “5”, etc.

#### Validation: All whole colony counts should have plot combination = -1
#### This fails as all plot combinations for colony counts are NA
b1_abundance_data_ices %>%
  dplyr::filter(sample_breeding == 2) %>%
  assertr::assert(assertr::in_set(-1, allow.na = FALSE), plot_combination,
                  success_fun = assertr::success_logical, error_fun = assertr::just_warn)

#### Validation: All plot counts should have a positive plot combination value
#### Plots counts are only supplied by GB
#### This fails as all 2966 plot combinations for GB were not exported to ICES
b1_abundance_data_ices %>%
  dplyr::filter(sample_breeding == 1) %>%
  assertr::assert(assertive::assert_all_are_positive, plot_combination,
                  success_fun = assertr::success_logical, error_fun = assertr::just_warn)

## ----- count_flag, count_unit -----
### count_flag indicating if the count was breeding data or non breeding data (overwintering or migratory birds)
### count_unit is the method used to count birds, breeding birds are either counts of individuals (1) or pairs (2),
###  non breeding (wintering) bird counts are counts of individuals (1)
### standardise count units

#### Validation: all species non breeding counts should be for individuals (count_unit = 1)
b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "non_breeding") %>%
  assertr::assert(assertr::in_set(1), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: 11 species should have no breeding counts
species_no_breeding_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "not breeding") %>%
  dplyr::pull(aphia_id)

b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data") %>%
  assertr::assert(assertr::in_set(species_no_breeding_counts, inverse = TRUE), aphia_id,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: 5 species should have individual breeding counts
#### FAILS as there are pair breeding counts for Razorbill, Black Guillemot, Brunnich's Guillemot and Common Guillemot
species_individual_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "individual") %>%
  dplyr::pull(aphia_id)

b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_individual_counts) %>%
  assertr::assert(assertr::in_set(1, 4), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### standardise Black Guillemot pair counts to individual counts
#### Validation:: Black Guillemot conversion individual = 2 x pairs
species_aphia_id <- c(137130L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("2*pairs"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(count = dplyr::case_when(
    aphia_id == species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~ count * 2,
    TRUE ~ count
  ),
  ## Converted count_unit to 4 to indicate conversion from pairs to individuals
  count_unit = dplyr::case_when(
    aphia_id == species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2  ~ 4,
    TRUE ~ count_unit
  ))

#### standardise Razorbill and Common Guillemot pair counts to individual counts
#### Validation:: Razorbill and Common Guillemot conversion individual = pairs / 0.67
species_aphia_id <- c(137128L, 137133L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("pairs/0.67"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(count = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~ round(count / 0.67, 0),
    TRUE ~ count
  ),
  ## Converted count_unit to 4 to indicate conversion from pairs to individuals
  count_unit = dplyr::case_when(
    aphia_id %in% species_aphia_id
    & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~ 4,
    TRUE ~ count_unit
  ))

#### standardise Brunnichs Guillemot pair counts to individual counts
#### Pair counts occur in Iceland but no conversion factor provided.

#### Check Validation again: 5 species now should have individual breeding counts
#### FAILS as Brunnichs Guillemot' pair counts in Iceland not converted but counts too sparse anyway
b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_individual_counts) %>%
  assertr::assert(assertr::in_set(1, 4), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::just_warn)

#### Validation: 83 species should have pair breeding counts
#### FAILS as there are individual breeding counts for 78 species
species_pair_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "pairs") %>%
  dplyr::pull(aphia_id)

b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_pair_counts) %>%
  assertr::assert(assertr::in_set(2, 3), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### standardise Tern individual counts to pair counts
#### Validation:: Tern species conversion pairs = individuals / 1.5
species_aphia_id <- c(137162L, 137165L, 137166L, 567480L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("individuals/1.5"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(count = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 1 ~ round(count / 1.5, 0),
    TRUE ~ count
  ),
  ## Converted count_unit to 3 to indicate conversion from individuals to pairs
  count_unit = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 1 ~ 3,
    TRUE ~ count_unit
  ))

#### standardise Gull and Fulmar individual counts to pair counts
#### Slender-bill Gull added 2021 analysis all pair counts no count conversion required.
#### Validation:: Gull and Fulmar species conversion pairs = individuals / 2
species_aphia_id <- c(137195L, 567449L, 137138L, 137141L, 137142L, 137145L, 137146L, 137147L, 232052L, 137154L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("individuals/2"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(count = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 1 ~ round(count / 2, 0),
    TRUE ~ count
  ),
  ## Converted count_unit to 3 to indicate conversion from individuals to pairs
  count_unit = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 1 ~ 3,
    TRUE ~ count_unit
  ))

#### standardise Puffin individual counts to pair counts
#### Validation:: Puffin conversion pairs = individuals
species_aphia_id <- c(137131L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("individuals=pairs"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  ## Converted count_unit to 3 to indicate conversion from individuals to pairs
  dplyr::mutate(count_unit = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 1 ~ 3,
    TRUE ~ count_unit
  ))

#### exclude seabird species breeding individual counts
#### Validation:: Seabird breeding individual counts
species_aphia_id <- c(158939L, 158941L, 158943L, 159168L, 148791L, 232037L, 159164L,
                      159172L, 159176L, 159197L, 159047L, 159052L, 137194L, 212611L,
                      212619L, 137137L, 465358L, 137071L, 159089L, 159090L, 159115L,
                      232054L, 137186L, 137187L, 137188L, 147436L, 212609L, 137189L,
                      159034L, 159035L, 137072L, 137073L, 232039L, 159097L, 159098L,
                      148776L, 159037L, 159040L, 137192L, 137178L, 137179L, 137168L,
                      137169L, 159046L, 416678L, 159135L, 159137L, 137181L, 137182L,
                      137183L, 137203L, 212721L, 137156L, 137074L, 137171L, 137172L,
                      137173L, 137174L, 137160L, 232042L, 159029L, 225916L, 158968L,
                      158970L, 159142L, 147431L, 159177L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("Exclude individuals"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::filter(!(aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data"
                  & count_unit == 1))

#### Check Validation again. 83 species should now have pair breeding counts
b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_pair_counts) %>%
  assertr::assert(assertr::in_set(2, 3), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

### exclude colonies with infrequent counts
#### exclude colonies (except Fulmar) recorded less than two years

### CEMP guidelines(page 10): The minimum number of years of counts for a particular species
#### required for a colony or site to be included in the analysis should be set at two for all species
#### except northern fulmar (Fulmarus glacialis), which should be set at a minimum of five years
#### (ICES, 2010, 2011)
colony_frequent_except_fulmar <- b1_abundance_data_ices %>%
  dplyr::filter(aphia_id != 137195) %>%
  dplyr::distinct(ices_colony_id, year) %>%
  dplyr::count(ices_colony_id) %>%
  dplyr::filter(n >= 2) %>%
  dplyr::select(ices_colony_id)

#### exclude Fulmar colonies recorded over less than five years
colony_frequent_fulmar <- b1_abundance_data_ices %>%
  dplyr::filter(aphia_id == 137195) %>%
  dplyr::distinct(ices_colony_id, year) %>%
  dplyr::count(ices_colony_id) %>%
  dplyr::filter(n >= 5) %>%
  dplyr::select(ices_colony_id)

colony_frequent <- colony_frequent_except_fulmar %>%
  dplyr::bind_rows(colony_frequent_fulmar) %>%
  dplyr::distinct()

b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::filter(ices_colony_id %in% dplyr::pull(colony_frequent, ices_colony_id))

#### Validation: breeding_data should all have been a breeding season count (time period = 3)
b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data") %>%
  assertr::assert(assertr::in_set(3), time_period,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: non_breeding_data should all have been either a January peak count (time period = 1)
#### or Full Year mean count (July-June) (time period = 2)
b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "non_breeding") %>%
  assertr::assert(assertr::in_set(1, 2), time_period,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

## split abundance count data into analysis and deleted datasets
b1_abundance_data_ices <- b1_abundance_data_ices %>%
  dplyr::mutate(common_name_id = stringr::str_replace_all(common_name, "[:space:]|-", "_")) %>%
  dplyr::left_join(count_status %>%
                     dplyr::rename(colony_ices_label = ospar,
                                   common_name_id = common_name)) %>%
  dplyr::select(-id, -common_name_id)

#### Validation: all counts should have a data status
b1_abundance_data_ices %>%
  assertr::assert(assertr::not_na, data,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

## remove previous locally cached data
pins::pin_remove("b1_abundance_data_ices_analysis", board = "local")
pins::pin_remove("b1_abundance_data_ices_excluded", board = "local")

## b1_abundance_data_ices_analysis
b1_abundance_data_ices_analysis <- b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_detect(data, "analysis")) %>%
  dplyr::select(-data, -comment)
pins::pin(b1_abundance_data_ices_analysis, "b1_abundance_data_ices_analysis")

### species with insufficient or absent counts in year range for the OSPAR region / sub-region
b1_abundance_data_ices_excluded <- b1_abundance_data_ices %>%
  dplyr::filter(stringr::str_detect(data, "analysis", negate = TRUE)) %>%
  dplyr::select(-data, -comment)
pins::pin(b1_abundance_data_ices_excluded, "b1_abundance_data_ices_excluded")




