# set config
Sys.setenv(R_CONFIG_ACTIVE = "production")

# network_path
directory_path <- config::get("directory_path")

## species table ---------------------------------------------------------------

### path
species_path <- fs::path(directory_path, "Lookup/bh1_lookup_tables_20220407.xlsx")

### species table
species <- readxl::read_xlsx(species_path, sheet = "species") %>%
  janitor::clean_names() %>%
  dplyr::rename(functional_group = fun_group)

# Replace unicode characters and apostrophes
species <- species %>%
  dplyr::mutate(common_name = dplyr::case_when(
    aphia_id == 159088 ~ "Bewicks swan",
    aphia_id == 137134 ~ "Brunnichs guillemot",
    aphia_id == 137194 ~ "Corys shearwater",
    aphia_id == 137192 ~ "Leachs storm-petrel",
    aphia_id == 232041 ~ "Stellers eider",
    TRUE ~ common_name)
  )

### save
usethis::use_data(species, overwrite = TRUE)

## country table ---------------------------------------------------------------

### path
country_path <- fs::path(directory_path, "Lookup/bh1_lookup_tables.xlsx")

### country table
country <- readxl::read_xlsx(country_path, sheet = "country")

### save
usethis::use_data(country, overwrite = TRUE)

## regional weightings ---------------------------------------------------------

### path
regional_weightings_path <- fs::path(directory_path, "Lookup/bh1_lookup_tables_20220407.xlsx")

### regional_weightings
regional_weightings <- readxl::read_xlsx(regional_weightings_path, sheet = "regional_weightings") %>%
  janitor::clean_names() %>%
  dplyr::mutate(colony_ices_label = stringr::str_glue("{as.roman(ospar_region)}{tidyr::replace_na(ospar_subregion, '')}")) %>%
  dplyr::relocate(colony_ices_label, .after = ospar_subregion) %>%
  dplyr::mutate(species_name = dplyr::recode(species_name,
                                             `rissa tridactyla` = "Rissa tridactyla",
                                             `morus bassanus` = "Morus bassanus")) %>%
  dplyr::select(-aphia_id)

## ----- aphia_id, scientific_name, common_name -----
### standardize species names
regional_weightings <- regional_weightings %>%
  dplyr::left_join(species %>%
                     dplyr::select(aphia_id, tidyselect::contains("name")),
                   by = c("species_name" = "scientific_name")) %>%
  dplyr::relocate(aphia_id, .before = species_name) %>%
  dplyr::relocate(common_name, .after = species_name)

#### Validation: All species names should have a matching aphia_id
#### FAILS: Greylag Goose Anser anser included in Norway but in the species table
regional_weightings %>%
  assertr::assert(assertr::not_na, aphia_id,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Remove Greylag Goose Anser anser
regional_weightings <- regional_weightings %>%
  dplyr::filter(species_name != "Anser anser")

#### Validation: Now all species names should have a matching aphia_id
regional_weightings %>%
  assertr::assert(assertr::not_na, aphia_id,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: All source year weighting should be integers
regional_weightings %>%
  dplyr::filter(!is.na(source_year_weighting)) %>%
  assertr::assert(assertive::is_whole_number, source_year_weighting,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

## change source_year_weighting to integer
regional_weightings <- regional_weightings %>%
  dplyr::mutate(source_year_weighting = as.integer(source_year_weighting))

## ----- weighting_value, count_flag, count_unit -----
### count_flag indicating if the count was breeding data or non breeding data (overwintering or migratory birds)
### count_unit is the method used to count birds, breeding birds are either counts of individuals (1) or pairs (2),
###  non breeding (wintering) bird counts are counts of individuals (1)
### standardise count units

### exclude zero weighting values
regional_weightings <- regional_weightings %>%
  dplyr::filter(weighting_value != 0)

#### Validation: 11 species should have no breeding regional weighting entries
species_no_breeding_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "not breeding") %>%
  dplyr::pull(aphia_id)

regional_weightings %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data") %>%
  assertr::assert(assertr::in_set(species_no_breeding_counts, inverse = TRUE), aphia_id,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: 5 species should have individual breeding counts
#### FAILS as there are pair breeding counts for Razorbill, Black Guillemot, Brunnichs guillemot and Common Guillemot
species_individual_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "individual") %>%
  dplyr::pull(aphia_id)

regional_weightings %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_individual_counts) %>%
  assertr::assert(assertr::in_set(1, 4), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### standardise Black Guillemot pair counts to individual counts
#### Validation:: Black Guillemot conversion individual = 2 x pairs
species_aphia_id <- c(137130L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("2*pairs"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

regional_weightings <- regional_weightings %>%
  dplyr::mutate(weighting_value = dplyr::case_when(
    aphia_id == species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~ weighting_value * 2,
    TRUE ~ weighting_value
  ),
  #### Converted count_unit to 4 to indicate conversion from pairs to individuals
  count_unit = dplyr::case_when(
    aphia_id == species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2  ~ 4,
    TRUE ~ count_unit
  ))

#### standardise Razorbill and Common Guillemot pair counts to individual counts
#### Validation:: Razorbill and Common Guillemot conversion individual = pairs / 0.67
species_aphia_id <- c(137128L, 137133L)
species %>%
  dplyr::filter(aphia_id %in% species_aphia_id) %>%
  assertr::assert(assertr::in_set("pairs/0.67"), conversion,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

regional_weightings <- regional_weightings %>%
  dplyr::mutate(weighting_value = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~
      round(weighting_value / 0.67, 0),
    TRUE ~ weighting_value
  ),
  #### Converted count_unit to 4 to indicate conversion from pairs to individuals
  count_unit = dplyr::case_when(
    aphia_id %in% species_aphia_id & stringr::str_to_lower(count_flag) == "breeding_data" & count_unit == 2 ~ 4,
    TRUE ~ count_unit
  ))

#### standardise Brunnichs Guillemot pair counts to individuals
#### Unable to convert as no conversion criteria in species table, however Norway
#### have not supplied any breeding counts so can exclude
regional_weightings <- regional_weightings %>%
  dplyr::filter(aphia_id != 137134)

#### Validation: 5 species should now have individual breeding counts
regional_weightings %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_individual_counts) %>%
  assertr::assert(assertr::in_set(1, 4), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

#### Validation: 80 species should have pair breeding counts
#### All breeding regional weightings are for pairs and so no conversions required
species_pair_counts <- species %>%
  dplyr::filter(stringr::str_to_lower(required_count_unit) == "pairs") %>%
  dplyr::pull(aphia_id)

regional_weightings %>%
  dplyr::filter(stringr::str_to_lower(count_flag) == "breeding_data",
                aphia_id %in% species_pair_counts) %>%
  assertr::assert(assertr::in_set(2, 3), count_unit,
                  success_fun = assertr::success_logical, error_fun = assertr::error_stop)

### format regional_weightings
regional_weightings <- regional_weightings %>%
  dplyr::select(-common_name)

### save
usethis::use_data(regional_weightings, overwrite = TRUE)

## count_status table ----------------------------------------------------------
count_status_path <- fs::path(directory_path, "Lookup/bh1_lookup_tables_20220525.xlsx")

count_status <- readxl::read_xlsx(count_status_path, sheet = "count_status")

### save
usethis::use_data(count_status, overwrite = TRUE)






